<?php

namespace Ulmus\User\Authorize\Bearer;

enum JsonWebTokenAlgorithmEnum
{
    case HS256;

    case HS384;

    case HS512;

    case RS256;

    case RS384;

    case RS512;

    case ES256;

    case ES384;

    case ES512;

    case PS256;

    case PS384;

    case PS512;

    public static function list() : array
    {
        return array_map(fn(JsonWebTokenAlgorithmEnum $case) => $case->name, static::cases());
    }

    public static function fromString(string $name) : self
    {
        foreach (static::cases() as $item) {
            if ($item->name === $name) {
                return $item;
            }
        }
    }

    public static function exists(string $key) : bool
    {
        return in_array($key, static::list());
    }

    public function assessOperability() : void
    {
        $method = $this->phpAlgoMethods();

        if (empty($method) || ! in_array($method[0], $method[2])) {
            throw new JsonWebTokenException(sprintf("Choosen algorithm do not seems to be supported by your PHP version '%s'", $this->name));
        }
    }

    public function phpAlgoMethods() : false|array
    {
        return match($this) {
            self::HS256 => [ 'sha256', 'hash_hmac', hash_hmac_algos() ],
            self::HS384 => [ 'sha384', 'hash_hmac', hash_hmac_algos() ],
            self::HS512 => [ 'sha512', 'hash_hmac', hash_hmac_algos() ],
            # Support for other algorithms not implemented yet...
        } ?? false;
    }
}
