<?php

namespace Ulmus\User\Authorize\Bearer;

class JsonWebTokenEncoder
{
    protected string $token;

    protected array $header = [
        "typ" => "JWT",
    ];

    public function __construct(
        public array $payload,
        public string $secretKey,
        protected JsonWebTokenAlgorithmEnum $algorithm = JsonWebTokenAlgorithmEnum::HS256,
    ) {
        $this->header['alg'] = $this->algorithm->name;
    }

    public static function base64url_encode($data) : string
    {
        return rtrim(strtr(base64_encode($data), '+/', '-_'), '=');
    }

    public function encode() : string
    {
        $jsonHeader = json_encode($this->header);
        $jsonPayload = json_encode($this->payload);

        $encodedHeader = static::base64url_encode($jsonHeader);
        $encodedPayload = static::base64url_encode($jsonPayload);

        list($algo, $method, ) = $this->algorithm->phpAlgoMethods();

        switch($method) {
            case 'hash_hmac':
                $signature = hash_hmac($algo, sprintf("%s.%s", $encodedHeader, $encodedPayload), $this->secretKey, true);
                break;
        }

        $this->token = sprintf("%s.%s.%s", $encodedHeader, $encodedPayload, static::base64url_encode($signature));

        return $this->getToken();
    }

    public function getToken() : string
    {
        return $this->token;
    }
}