<?php

namespace Ulmus\User\Authorize\Header;

use Psr\Http\Message\ServerRequestInterface;
use Ulmus\User\Authorize\Bearer\JsonWebToken;
use Ulmus\User\Authorize\Bearer\JsonWebTokenDecoder;
use Ulmus\User\Entity\UserInterface;
use Ulmus\User\Lib\Authorize;

class BearerMethod implements MethodInterface
{
    protected JsonWebTokenDecoder $jwt;

    public function __construct(
        protected UserInterface $user,
        protected string $token
    ) {}

    public function execute(ServerRequestInterface $request) : bool
    {
        switch($this->autodetectTokenType()) {
            case BearerTokenTypeEnum::JsonWebToken:
                $authorize = new Authorize($this->user);

                $payload = $this->jwt->getPayload();

                if ($payload['sub'] ?? false) {
                    if ( ! $authorize->logUser((int) $payload['sub']) ) {
                        throw new \Exception("Given user id do not match with an existing/active user");
                    }
                }
                else {
                    throw new \InvalidArgumentException("Given JsonWebToken is missing a 'sub' key (which concords to user id)");
                }

                break;

            case BearerTokenTypeEnum::UniqueKey:
                # @TODO
                break;
        }


        return $this->user->loggedIn();
    }

    public function autodetectTokenType() : BearerTokenTypeEnum
    {
        $this->jwt = new JsonWebTokenDecoder($this->token, getenv('LEAN_RANDOM'));

        if ( $this->jwt->isJWT() ) {
            return BearerTokenTypeEnum::JsonWebToken;
        }

        return BearerTokenTypeEnum::UniqueKey;
    }

    protected function userFromJWT() : UserInterface
    {

    }
}